import { Card, Checkbox, Col, Row, Select, Spin } from "antd";
import { useEffect, useRef, useState } from "react";
import { Constants, myFetch } from "../../utils";
import { useTranslation } from "react-i18next";
import { Virtuoso } from "react-virtuoso";

export default function LogCard({
  type,
  title,
  customResult,
  logApiAddress = Constants.LOG_API_ADDRESS,
}) {
  const { t } = useTranslation();
  // const [checkboxAutoScrollChecked, setCheckboxAutoScrollChecked] =
  // useState(true);
  const [checkboxInfoChecked, setCheckboxInfoChecked] = useState(true);
  const [checkboxWarningChecked, setCheckboxWarningChecked] = useState(true);
  const [checkboxErrorChecked, setCheckboxErrorChecked] = useState(true);
  const virtuosoRef = useRef();

  const [loadingSpinner, setLoadingSpinner] = useState(true);
  const [availableDates, setAvailableDates] = useState([]);
  const [selectedDate, setSelectedDate] = useState("");
  const [logs, setLogs] = useState([]);

  const sseEventSource = useRef(null);

  useEffect(() => {
    if (type === "") return;

    // init request to get all available dates
    myFetch(`s/${type}`, "GET", null, {}, 0, logApiAddress)
      .then((data) => {
        let dates = [];

        if (data.length === 0) return;

        dates = data.map((date) => {
          return {
            value: date,
            label: date,
          };
        });

        setAvailableDates(dates);
        setSelectedDate(dates[0].value);

        if (selectedDate === dates[0].value) {
          loadLogs(dates[0].value);
        }
      })
      .catch((errStatus) => console.log("error", errStatus));
  }, [type]);

  const loadLogs = (date) => {
    setLoadingSpinner(true);

    if (date === "") return;

    // fetch logs
    myFetch(`s/${type}?d=${date}`, "GET", null, {}, 0, logApiAddress)
      .then((data) => {
        setLoadingSpinner(false);

        // data is undefined when there are no logs for the selected date (e.g. log was deleted)
        if (data === undefined) {
          setLogs([]);
          return;
        }

        setLogs(data);
      })
      .catch((errStatus) => console.log("error", errStatus));
  };

  useEffect(() => loadLogs(selectedDate), [selectedDate]);

  useEffect(() => {
    if (type === "" || selectedDate === "") return;

    const currentDate = new Date();

    const today = `${currentDate.getDate()}-${
      currentDate.getMonth() + 1
    }-${currentDate.getFullYear()}`;

    // only open the sse connection if the selected date is today
    if (selectedDate === today) {
      sseEventSource.current = new EventSource(
        `${logApiAddress}/sse/${type}/${selectedDate}`
      );

      sseEventSource.current.onmessage = (event) => {
        const data = JSON.parse(event.data);

        setLogs((prevLogs) => {
          const newLogs = [...prevLogs];

          data.forEach((log) => {
            newLogs.push(log);
          });

          return newLogs;
        });
      };

      sseEventSource.current.onerror = (event) =>
        console.log("sse error", event);

      sseEventSource.current.onopen = (event) => console.log("sse open", event);

      sseEventSource.current.onclose = (event) =>
        console.log("sse close", event);

      return () => sseEventSource.current.close();
    }
  }, [type, selectedDate]);

  /*
  useEffect(() => {
    // scroll to bottom
    if (
      checkboxAutoScrollChecked &&
      virtuosoRef.current !== undefined &&
      virtuosoRef.current.scrollToIndex !== null
    ) {
      console.log("scrolltobottom", logs.length);
      /*
      virtuosoRef.current.scrollToIndex({
        index: logs.length - 1,
        align: "start",
        behavior: "auto",
      });*/ /*
    }
  }, [logs]); */

  /*
  <Col>
    <Checkbox
      checked={checkboxAutoScrollChecked}
      onChange={(e) =>
        setCheckboxAutoScrollChecked(e.target.checked)
      }
    >
      {t("logCard.card.checkbox.autoScroll")}
    </Checkbox>
  </Col> 
  */

  const filteredLogs = logs.filter((log) => {
    const logType = log.charAt(0);

    // some other logs which have no type defined
    if (logType !== "I" && logType !== "W" && logType !== "E") {
      return true;
    }

    return (
      (checkboxInfoChecked && logType === "I") ||
      (checkboxWarningChecked && logType === "W") ||
      (checkboxErrorChecked && logType === "E")
    );
  });

  return (
    <Card
      title={
        <Row>
          <Col xs={24} md={{ span: 10, offset: 0 }}>
            {title}
          </Col>

          <Col xs={24} xl={{ span: 10, offset: 4 }}>
            <Row xs={24} justify="end" align="middle">
              <Col>
                <Checkbox
                  disabled={customResult !== undefined}
                  checked={checkboxInfoChecked}
                  onChange={(e) => setCheckboxInfoChecked(e.target.checked)}
                >
                  {t("logCard.card.checkbox.info")}
                </Checkbox>
              </Col>
              <Col>
                <Checkbox
                  checked={checkboxWarningChecked}
                  onChange={(e) => setCheckboxWarningChecked(e.target.checked)}
                >
                  {t("logCard.card.checkbox.warning")}
                </Checkbox>
              </Col>
              <Col>
                <Checkbox
                  disabled={customResult !== undefined}
                  checked={checkboxErrorChecked}
                  onChange={(e) => setCheckboxErrorChecked(e.target.checked)}
                >
                  {t("logCard.card.checkbox.error")}
                </Checkbox>
              </Col>

              <Col>
                <Select
                  disabled={customResult !== undefined}
                  style={{ width: 120 }}
                  options={availableDates}
                  value={selectedDate}
                  onChange={(value) => setSelectedDate(value)}
                />
              </Col>
            </Row>
          </Col>
        </Row>
      }
      style={{
        height: "90vh",
        overflow: "hidden",
        overflowWrap: "break-word",
        width: "100%",
      }}
    >
      {customResult === undefined ? (
        <>
          {!loadingSpinner && type !== "" ? (
            <div style={{ height: "80vh", paddingRight: 6 }}>
              <Virtuoso
                ref={virtuosoRef}
                initialTopMostItemIndex={logs.length - 1}
                data={filteredLogs}
                itemContent={(_, log) => {
                  const logType = log.charAt(0);
                  let style = {};
                  let color = "";

                  if (checkboxInfoChecked && logType === "I") {
                    color = "#44bd32";
                  } else if (checkboxWarningChecked && logType === "W") {
                    color = "#e67e22";
                  } else if (checkboxErrorChecked && logType === "E") {
                    color = "#e74c3c";
                  } else {
                    color = "#2980b9"; // no match
                  }

                  style = { padding: "1px", color };

                  return <span style={style}>{log}</span>;
                }}
                followOutput={true}
              />
            </div>
          ) : (
            <div
              style={{
                display: "flex",
                justifyContent: "center",
                alignItems: "center",
                height: "80vh",
              }}
            >
              <Spin size="large"></Spin>
            </div>
          )}
        </>
      ) : (
        <div
          style={{
            display: "flex",
            justifyContent: "center",
            alignItems: "center",
            height: "80vh",
          }}
        >
          {customResult}
        </div>
      )}
    </Card>
  );
}

/*

  useEffect(() => {
    console.log("useffect logs");

    // scroll to bottom
    if (checkboxAutoScrollChecked && virtuosoRef.current !== undefined) {
      console.log("logLength current", logs.length);

      //virtuosoRef.current.autoscrollToBottom();

      virtuosoRef.current.scrollToIndex({
        index: logs.length + 1,
        align: "start",
        behavior: "auto",
      });
    }
  }, [logs]);

 <Tooltip title={t("logCard.tooltip.reload")}>
            <ReloadOutlined onClick={() => loadLogs(selectedDate)} />
          </Tooltip>
*/

/*
  const inBrackets = (content, contentColor) => {
    return (
      <>
        <span style={{ color: "#95a5a6" }}>[</span>
        <span style={{ color: contentColor }}>{content}</span>
        <span style={{ color: "#95a5a6" }}>] </span>
      </>
    );
  };

  const getMessageType = (logType) => {
    if (logType === Constants.SYSTEM_LOG_TYPE.INFO) {
      return inBrackets("INFO", "#27ae60");
    }
    if (logType === Constants.SYSTEM_LOG_TYPE.DEBUG) {
      return inBrackets("DEB", "#e67e22");
    }
    if (logType === Constants.SYSTEM_LOG_TYPE.ERROR) {
      return inBrackets("ERR", "#e74c3c");
    }
    return "";
  };

  {logData.Logs.filter((log) => {
            return (
              (log.Type === 0 && checkboxInfoChecked) ||
              (log.Type === 1 && checkboxErrorChecked)
            );
          }).map((log, i) => {
            if (
              (log.Type === 0 && !checkboxInfoChecked) ||
              (log.Type === 1 && !checkboxErrorChecked)
            ) {
              return "";
            }

            return (
              <div key={i}>
                {inBrackets(FormatDatetime(log.Time), "#7f8c8d")}{" "}
                {getMessageType(log.Type)}
                {log.Message}
              </div>
            );
          })}
*/

/*myFetch(
      `/log?type=${type === "grouptasks" ? "g" : "s"}&date=${date}&lang=${
        i18n.language
      }`,
      "GET",
      null,
      {},
      0,
      Constants.LOG_API_ADDRESS
    )
      .then((data) => {
        fetchResponse.current = data;

        setLoadingSpinner(false);

        if (data.Logs === null) {
          data.Logs = [];
        } else {
          for (let i = 0; i < data.Logs.length; i++) {
            let items = [];

            if (data.Logs[i].Message === "") {
              items.push(
                <span style={{ color: "#e74c3c" }}>
                  Message not found.{" "}
                  {data.Logs[i].LogData.length > 0 && (
                    <>
                      Given Args:{" "}
                      {data.Logs[i].LogData.map((data) => (
                        <span>
                          {data.Type}:{data.Value}
                        </span>
                      ))}
                    </>
                  )}
                </span>
              );
            }

            let splittedMessage = data.Logs[i].Message.split(" ");

            for (let s = 0; s < splittedMessage.length; s++) {
              if (splittedMessage[s].includes("%")) {
                if (splittedMessage[s] === "%userId%") {
                  const foundData = data.Logs[i].LogData.find(
                    (data) => data.Type === "userId"
                  );

                  if (foundData !== undefined) {
                    const foundUser = fetchResponse.current.Users.find(
                      (user) => user.Id === foundData.Value
                    );

                    items.push(
                      <span key={s} style={{ color: Constants.COLORS.PRIMARY }}>
                        {foundUser !== undefined
                          ? foundUser.Username
                          : foundData.Value}{" "}
                      </span>
                    );
                  } else {
                    items.push(
                      <ColoredSpanItem
                        key={s}
                        dataLogs={data.Logs[i]}
                        splittedMessage={splittedMessage[s]}
                      />
                    );
                  }
                } else if (splittedMessage[s] === "%groupTaskId%") {
                  const foundData = data.Logs[i].LogData.find(
                    (data) => data.Type === "groupTaskId"
                  );

                  if (foundData !== undefined) {
                    const foundGroupTask =
                      fetchResponse.current.GroupTasks.find(
                        (groupTask) => groupTask.Id === foundData.Value
                      );

                    items.push(
                      <ColoredSpanItem
                        key={s}
                        dataLogs={data.Logs[i]}
                        splittedMessage={splittedMessage[s]}
                        children={
                          <Popover
                            content={
                              <div>
                                <span style={{ fontWeight: "bold" }}>ID: </span>
                                <Link
                                  to={`${Constants.ROUTE_PATHS.GROUP_TASKS_VIEW}${foundData.Value}`}
                                >
                                  {foundData.Value}
                                </Link>
                              </div>
                            }
                            title={t("logCard.popover.groupTaskId.title")}
                            trigger="click"
                          >
                            <span style={{ cursor: "pointer" }}>
                              {foundGroupTask !== undefined
                                ? foundGroupTask.GroupName
                                : foundData.Value}
                            </span>
                          </Popover>
                        }
                      />
                    );
                  } else {
                    items.push(
                      <ColoredSpanItem
                        key={s}
                        dataLogs={data.Logs[i]}
                        splittedMessage={splittedMessage[s]}
                      />
                    );
                  }
                } else if (splittedMessage[s] === "%taskStepId%") {
                  const foundData = data.Logs[i].LogData.find(
                    (data) => data.Type === "taskStepId"
                  );

                  if (foundData !== undefined) {
                    const foundGroupTaskStep =
                      fetchResponse.current.GroupTasksSteps.find(
                        (step) => step.Id === foundData.Value
                      );

                    items.push(
                      <ColoredSpanItem
                        key={s}
                        dataLogs={data.Logs[i]}
                        splittedMessage={splittedMessage[s]}
                        children={
                          <Popover
                            content={
                              <div>
                                <span style={{ fontWeight: "bold" }}>ID: </span>
                                <span>
                                  {foundGroupTaskStep !== undefined
                                    ? foundGroupTaskStep.Id
                                    : foundData.Value}
                                </span>
                              </div>
                            }
                            title={t("logCard.popover.groupTaskStep.title")}
                            trigger="click"
                          >
                            {" "}
                            <span style={{ cursor: "pointer" }}>
                              {foundGroupTaskStep !== undefined
                                ? getGroupTaskStepName(foundGroupTaskStep)
                                : foundData.Value}
                            </span>
                          </Popover>
                        }
                      />
                    );
                  } else {
                    items.push(
                      <ColoredSpanItem
                        key={s}
                        dataLogs={data.Logs[i]}
                        splittedMessage={splittedMessage[s]}
                      />
                    );
                  }
                } else if (splittedMessage[s] === "%roleId%") {
                  const foundData = data.Logs[i].LogData.find(
                    (data) => data.Type === "roleId"
                  );

                  if (foundData !== undefined) {
                    const foundRole = fetchResponse.current.Roles.find(
                      (role) => role.Id === foundData.Value
                    );

                    items.push(
                      <ColoredSpanItem
                        key={s}
                        dataLogs={data.Logs[i]}
                        splittedMessage={splittedMessage[s]}
                        children={
                          <Popover
                            content={
                              <div>
                                <span style={{ fontWeight: "bold" }}>ID: </span>
                                <span>{foundData.Value}</span>
                              </div>
                            }
                            title={t("logCard.popover.role.title")}
                            trigger="click"
                          >
                            {" "}
                            <span style={{ cursor: "pointer" }}>
                              {foundRole !== undefined
                                ? foundRole.DisplayName
                                : foundData.Value}
                            </span>
                          </Popover>
                        }
                      />
                    );
                  } else {
                    items.push(
                      <ColoredSpanItem
                        key={s}
                        dataLogs={data.Logs[i]}
                        splittedMessage={splittedMessage[s]}
                      />
                    );
                  }
                } else if (splittedMessage[s] === "%groupId%") {
                  const foundDataCategory = data.Logs[i].LogData.find(
                    (data) => data.Type === "category"
                  );

                  const foundDataGroupId = data.Logs[i].LogData.find(
                    (data) => data.Type === "groupId"
                  );

                  if (
                    foundDataCategory !== undefined &&
                    foundDataGroupId !== undefined
                  ) {
                    const foundGroup =
                      fetchResponse.current.CategoryGroups.find(
                        (categoryGroup) =>
                          categoryGroup.category === foundDataCategory.Value
                      ).groups.find(
                        (group) => group.id === foundDataGroupId.Value
                      );

                    items.push(
                      <ColoredSpanItem
                        key={s}
                        dataLogs={data.Logs[i]}
                        splittedMessage={splittedMessage[s]}
                        children={
                          <Popover
                            content={
                              <div>
                                <span style={{ fontWeight: "bold" }}>ID: </span>
                                <span>{foundDataGroupId.Value}</span>
                              </div>
                            }
                            title={t("logCard.popover.groupId.title")}
                            trigger="click"
                          >
                            {" "}
                            <span style={{ cursor: "pointer" }}>
                              {foundGroup !== undefined
                                ? foundGroup.name
                                : foundDataGroupId.Value}
                            </span>
                          </Popover>
                        }
                      />
                    );
                  } else {
                    items.push(
                      <ColoredSpanItem
                        key={s}
                        dataLogs={data.Logs[i]}
                        splittedMessage={splittedMessage[s]}
                      />
                    );
                  }
                } else {
                  items.push(
                    <ColoredSpanItem
                      key={s}
                      dataLogs={data.Logs[i]}
                      splittedMessage={splittedMessage[s]}
                    />
                  );
                }
              } else {
                items.push(<span key={s}>{splittedMessage[s]} </span>);
              }
            }

            data.Logs[i].Message = items;
          }
        }

        setLogData(data);
      })
      .catch((errStatus) => console.log("error", errStatus)); */

/*
  const getDate = () => {
    const today = new Date();

    return `${today.getDate()}-${today.getMonth() + 1}-${today.getFullYear()}`;
  }; */

/*const getColorCode = (index) => {
    const colorCodes = ["#3498db", "#9b59b6", "#1abc9c"];
    const colorIndex = index % colorCodes.length;
    return colorCodes[colorIndex];
  };

  const ColoredSpanItem = ({ dataLogs, splittedMessage, children }) => {
    const logDataIndex = dataLogs.LogData.findIndex(
      (data) => data.Type === splittedMessage.replace(new RegExp("%", "g"), "")
    );

    if (logDataIndex !== -1) {
      return (
        <span style={{ color: getColorCode(logDataIndex) }}>
          {children === undefined ? (
            dataLogs.LogData[logDataIndex].Value
          ) : (
            <>{children}</>
          )}{" "}
        </span>
      );
    }
    return <></>;
  }; 

  const getGroupTaskStepName = (groupTaskStep) => {
    const foundName = fetchResponse.current.CategoryGroups.find(
      (categoryGroup) =>
        categoryGroup.category ===
        fetchResponse.current.GroupTasks.find(
          (groupTask) => groupTask.Id === groupTaskStep.GroupTasksId
        )?.Category
    )?.groups.find(
      (groups) =>
        groups.id ===
        fetchResponse.current.GroupTasks.find(
          (groupTask) => groupTask.Id === groupTaskStep.GroupTasksId
        ).GroupId
    ).tasks[groupTaskStep.Step - 1];

    return foundName !== undefined ? foundName.name : groupTaskStep.Id;
  }; */

/*
{logData.Dates.findIndex((date) => date === selectedDate) > 0 ? (
            <Tooltip title={t("logCard.tooltip.previous")}>
              <ArrowLeftOutlined
                onClick={() =>
                  setSelectedDate(
                    logData.Dates[
                      logData.Dates.findIndex((date) => date === selectedDate) -
                        1
                    ]
                  )
                }
              />
            </Tooltip>
          ) : (
            <ArrowLeftOutlined
              style={{
                color: "rgba(0, 0, 0, 0.25)",
                cursor: "not-allowed",
              }}
            />
          )}

            {logData.Dates.findIndex((date) => date === selectedDate) + 1 <
          logData.Dates.length ? (
            <Tooltip title={t("logCard.tooltip.next")}>
              <ArrowRightOutlined
                onClick={() =>
                  setSelectedDate(
                    logData.Dates[
                      logData.Dates.findIndex((date) => date === selectedDate) +
                        1
                    ]
                  )
                }
              />
            </Tooltip>
          ) : (
            <ArrowRightOutlined
              style={{
                color: "rgba(0, 0, 0, 0.25)",
                cursor: "not-allowed",
              }}
            />
          )}
*/
