import {
  Button,
  Divider,
  Form,
  Input,
  InputNumber,
  Modal,
  Select,
  Space,
  Tag,
  notification,
} from "antd";
import { GetUuid, myFetch } from "../../../utils";
import { InfoCircleOutlined } from "@ant-design/icons";
import TextArea from "antd/es/input/TextArea";
import { useTranslation } from "react-i18next";
import { useWebSocketContext } from "../../../Contexts/WebSocketContext";
import { useGroupTasksContext } from "../../../Contexts/GroupTasksContext";
import { SentMessagesCommands } from "../../../Handlers/WebSocketMessageHandler";
import { useEffect, useRef, useState } from "react";

export default function GroupTypeSelectionModal({
  isOpen,
  setIsOpen,
  categoryGroup,
  currentSelectedModalGroupType,
  setCurrentSelectedModalGroupType,
}) {
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();
  const webSocketContext = useWebSocketContext();
  const groupTasksContext = useGroupTasksContext();
  const { t } = useTranslation();
  const handleCancel = () => setIsOpen(false);

  const isStartTaskPossible = () => {
    if (
      currentSelectedModalGroupType === null ||
      currentSelectedModalGroupType === undefined
    ) {
      return true;
    }

    let possible = false;

    categoryGroup.groups.forEach((group) => {
      if (group.id === currentSelectedModalGroupType) {
        if (group.tasks === null) {
          possible = true;
          return;
        }
      }
    });

    return possible;
  };

  const handleStartTask = () => {
    let canTaskBeStarted = true;

    const groupTaskDescription = document.getElementById(
      "fitem-grouptypedescription"
    ).value;

    if (groupTaskDescription === "") {
      canTaskBeStarted = false;
    }

    const userSpecifiedGlobalInputForm = document.getElementById(
      "groupTypeSelectionUserSpecifiedGlobalInputForm"
    );

    let globalInputs = [];
    let foundSelectInputs = 0;

    if (canTaskBeStarted && userSpecifiedGlobalInputForm !== null) {
      const userSpecifiedGlobalInputs =
        userSpecifiedGlobalInputForm.getElementsByTagName("input");

      if (userSpecifiedGlobalInputs.length > 0) {
        for (let i = 0; i < userSpecifiedGlobalInputs.length; i++) {
          if (userSpecifiedGlobalInputs[i].type !== "text") {
            foundSelectInputs = foundSelectInputs + 1;
            continue;
          }

          if (userSpecifiedGlobalInputs[i].value === "") {
            canTaskBeStarted = false;
            break;
          }

          globalInputs.push({
            parameterName: userSpecifiedGlobalInputs[i].id,
            value: userSpecifiedGlobalInputs[i].value,
          });
        }
      }

      const userSpecifiedGlobalTextareas =
        userSpecifiedGlobalInputForm.getElementsByTagName("textarea");

      if (userSpecifiedGlobalTextareas.length > 0) {
        for (let i = 0; i < userSpecifiedGlobalTextareas.length; i++) {
          if (userSpecifiedGlobalTextareas[i].value === "") {
            canTaskBeStarted = false;
            break;
          }

          globalInputs.push({
            parameterName: userSpecifiedGlobalTextareas[i].id,
            value: userSpecifiedGlobalTextareas[i].value,
          });
        }
      }
    }

    const selectInputsObjectKeys = Object.keys(groupTasksContext.selectInputs);

    // if there are select inputs, we need to check if they are all selected
    if (
      foundSelectInputs > 0 &&
      foundSelectInputs !== selectInputsObjectKeys.length
    ) {
      canTaskBeStarted = false;
    } else {
      selectInputsObjectKeys.forEach((key) => {
        globalInputs.push({
          parameterName: key,
          value: groupTasksContext.selectInputs[key].value,
          data: groupTasksContext.selectInputs[key].data,
        });
      });
    }

    if (!canTaskBeStarted) {
      notificationApi["error"]({
        message: t(
          "groupTasks.groupTypeSelectionModal.notification.globalInputsCannotBeEmpty.message"
        ),
        description: t(
          "groupTasks.groupTypeSelectionModal.notification.globalInputsCannotBeEmpty.description"
        ),
      });
      return;
    }

    setIsOpen(false);

    let groupName;
    let numberOfSteps;

    categoryGroup.groups.forEach((group) => {
      if (group.id === currentSelectedModalGroupType) {
        groupName = group.name;
        numberOfSteps = group.tasks.length;
        return;
      }
    });

    const rememberId = GetUuid();

    groupTasksContext.startGroupTasksOpenModalRememberIdRef.current =
      rememberId;

    webSocketContext.SendSocketMessage(SentMessagesCommands.StartGroupTasks, {
      category: categoryGroup.category,
      id: currentSelectedModalGroupType,
      groupName: groupName,
      description: groupTaskDescription,
      numberOfSteps: parseInt(numberOfSteps),
      globalInputs: globalInputs,
      rememberId: rememberId, // used to open the modal when group task is started by backend and send via websocket back
    });

    // reset select inputs
    groupTasksContext.setSelectInputs({});
  };

  return (
    <Modal
      title={t("groupTasks.groupTypeSelectionModal.title")}
      open={isOpen}
      onCancel={handleCancel}
      centered
      maskClosable={false}
      footer={[
        <Space key={0}>
          <Button key="back" onClick={handleCancel}>
            {t("common.button.cancel")}
          </Button>

          <Button
            key="submit"
            type="primary"
            disabled={isStartTaskPossible()}
            onClick={handleStartTask}
          >
            {t("groupTasks.groupTypeSelectionModal.button.startTask")}
          </Button>
        </Space>,
      ]}
    >
      {notificationContextHolder}
      <Select
        placeholder={t("groupTasks.groupTypeSelectionModal.select.placeholder")}
        style={{ width: "100%" }}
        value={currentSelectedModalGroupType}
        onChange={(value) => setCurrentSelectedModalGroupType(value)}
      >
        {categoryGroup.groups !== undefined
          ? categoryGroup.groups.map((group, index) => {
              return (
                <Select.Option key={index} value={group.id}>
                  {group.name}
                </Select.Option>
              );
            })
          : null}
      </Select>

      {currentSelectedModalGroupType !== null && (
        <Form layout="vertical">
          <br />
          <Form.Item
            label={t(
              "groupTasks.groupTypeSelectionModal.form.item.groupTypeDescription.label"
            )}
            tooltip={{
              title: t(
                "groupTasks.groupTypeSelectionModal.form.item.groupTypeDescription.tooltip.title"
              ),
              icon: <InfoCircleOutlined />,
            }}
            required
          >
            <Input id="fitem-grouptypedescription" />
          </Form.Item>
        </Form>
      )}

      <GroupGlobalInputs
        categoryGroup={categoryGroup}
        currentSelectedModalGroupType={currentSelectedModalGroupType}
        notificationApi={notificationApi}
        groupGlobalInputs={groupTasksContext.groupGlobalInputs}
        setGroupGlobalInputs={groupTasksContext.setGroupGlobalInputs}
      />
    </Modal>
  );
}

export function SelectMachineComponent({
  t,
  notificationApi,
  globalInput,
  onSelectChange,
  disabled,
}) {
  const groupTasksContext = useGroupTasksContext();
  const [loading, setLoading] = useState(false);
  const [options, setOptions] = useState([]);
  const responseData = useRef([]);

  useEffect(() => {
    // if globalInput.options is null, it means that user has not specified any options for this global input in the index.json
    if (globalInput.options === null) {
      notificationApi["error"]({
        message: t(
          "groupTasks.groupTypeSelectionModal.notification.globalInputOptionsMissing.message"
        ),
        description: t(
          "groupTasks.groupTypeSelectionModal.notification.globalInputOptionsMissing.description",
          {
            globalInputParamterName: globalInput.parameterName,
          }
        ),
      });

      return;
    }

    setLoading(true);

    myFetch("/machines", "POST", globalInput.options).then((data) => {
      const processedOptions = {};

      data.forEach((item, index) => {
        const partName = item.part_detail.Name;

        if (!processedOptions[partName]) {
          processedOptions[partName] = [];
        }

        let parsedNotes = "";

        if (item.Notes !== "") {
          try {
            parsedNotes = JSON.parse(item.Notes);
          } catch (error) {}
        }

        processedOptions[partName].push({
          key: index + item.part_detail.Name,
          label:
            // testing if Notes is empty string or if displayName is undefined
            parsedNotes === ""
              ? t("groupTasks.groupTypeSelectionModal.noNotesSpecified")
              : parsedNotes.displayName === undefined
              ? t(
                  "groupTasks.groupTypeSelectionModal.noNotesDisplayNameSpecified"
                )
              : parsedNotes.displayName,
          value: index,
          // disabling option if Notes is empty string or if displayName is undefined
          disabled: parsedNotes === "" || parsedNotes.displayName === undefined,
        });
      });

      const finalOptions = Object.keys(processedOptions).map(
        (groupName, index) => ({
          key: groupName + index,
          label: groupName,
          options: processedOptions[groupName],
        })
      );

      setOptions(finalOptions);
      responseData.current = data;
      setLoading(false);
    });
  }, []);

  const selectedInput =
    groupTasksContext.selectInputs[globalInput.parameterName]?.value;

  return (
    <Select
      disabled={disabled}
      loading={loading}
      placeholder={t(
        "groupTasks.groupTypeSelectionModal.select.machinePlaceholder"
      )}
      style={{ width: "100%" }}
      options={options}
      value={selectedInput}
      onSelect={(value) => {
        const parsedNotes = JSON.parse(responseData.current[value].Notes);

        groupTasksContext.setSelectInputs((prevState) => ({
          ...prevState,
          [globalInput.parameterName]: {
            value: parsedNotes.displayName,
            data: parsedNotes,
          },
        }));

        if (onSelectChange !== undefined) {
          onSelectChange(value, parsedNotes);
        }
      }}
    />
  );
}

function GroupGlobalInputs({
  categoryGroup,
  currentSelectedModalGroupType,
  notificationApi,
}) {
  const { t } = useTranslation();

  const getLabel = (displayName) => {
    return (
      <>
        {displayName}
        <Tag style={{ marginLeft: 6 }} color="purple">
          {t("groupTasks.tag.global")}
        </Tag>
      </>
    );
  };

  var elements = [];

  if (currentSelectedModalGroupType !== undefined) {
    categoryGroup.groups.forEach((group) => {
      if (
        group.id === currentSelectedModalGroupType &&
        group.globalInputs?.length > 0
      ) {
        group.globalInputs.forEach((globalInput, index) => {
          switch (globalInput.type) {
            case "text":
              elements.push(
                <Form.Item
                  key={index}
                  label={getLabel(globalInput.displayName)}
                  required
                >
                  <Input id={globalInput.parameterName} />
                </Form.Item>
              );
              break;
            case "number":
              elements.push(
                <Form.Item
                  key={index}
                  label={getLabel(globalInput.displayName)}
                  required
                >
                  <InputNumber
                    style={{ width: "100%" }}
                    max={Number.MAX_SAFE_INTEGER}
                    id={globalInput.parameterName}
                  />
                </Form.Item>
              );
              break;
            case "textarea":
              elements.push(
                <Form.Item
                  key={index}
                  label={getLabel(globalInput.displayName)}
                  required
                >
                  <TextArea id={globalInput.parameterName} />
                </Form.Item>
              );
              break;
            case "select_machine":
              elements.push(
                <Form.Item
                  key={index}
                  label={getLabel(globalInput.displayName)}
                  required
                >
                  <SelectMachineComponent
                    t={t}
                    notificationApi={notificationApi}
                    globalInput={globalInput}
                  />
                </Form.Item>
              );
              break;

            default:
              notificationApi["error"]({
                message: t(
                  "groupTasks.groupTypeSelectionModal.notification.globalInputTypeNotImplemented.message",
                  { globalInputType: globalInput.type }
                ),
                description: t(
                  "groupTasks.groupTypeSelectionModal.notification.globalInputTypeNotImplemented.description",
                  { globalInputDisplayName: globalInput.displayName }
                ),
              });
              break;
          }
        });
        return;
      }
    });
  }

  return (
    <>
      {elements.length > 0 ? (
        <div>
          <Divider />
          <h3 style={{ fontWeight: "bold" }}>
            {t("groupTasks.groupTypeSelectionModal.h3")}
          </h3>
          <Form
            layout="vertical"
            id="groupTypeSelectionUserSpecifiedGlobalInputForm"
          >
            {elements}
          </Form>
        </div>
      ) : null}
    </>
  );
}
